local addonName, addonData = ...;
local events = {};
local shortageList = {};

-- Variables
local WIDTH = 420;
local HEIGHT = 360;
local PADDING = 10;
local PADDING_TOP = 22;
local WIDTH_THIRD = WIDTH / 3 - PADDING;
local WIDTH_FOURTH = WIDTH / 4 - PADDING;
local WIDTH_FIFTH = WIDTH / 5 - PADDING;
local HEIGHT_NAV = 25;

local MAX_QUESTS = 25;

local FONT_SIZES = {
	small = 10,
	medium = 12
};

local editCurrencyId;
local selectedTalentTemplate;

-- Settings
local settingsFrame = CreateFrame("Frame", "DLUSettingsFrame");
local eventFrame = CreateFrame("Frame");

-- Notification System
CreateFrame("Frame", "DLUNotificationFrame");
DLUNotificationFrame:SetAlpha(0);
DLUNotificationFrame:SetWidth(320);
DLUNotificationFrame:SetHeight(80);
DLUNotificationFrame:SetPoint("TOP", UIParent, "TOP", 0, 0);
DLUNotificationFrame:CreateFontString("DLUNotification", "ARTWORK", "GameFontNormalLarge");
DLUNotification:SetPoint("CENTER", 0, 0);

-- Change Character UI
hooksecurefunc("PaperDollFrame_UpdateStats", function()
	DLU.showStats("player");
end)

-- UI related functions
local function achievementCompleted(achievementId)
	local _, _, _, _, _, _, _, _, _, _, _, _, wasEarnedByMe = GetAchievementInfo(achievementId);
	return wasEarnedByMe;
end

local function addToTracker(achievementId, button)
	if (achievementCompleted(achievementId)) then
		RemoveTrackedAchievement(achievementId);
		if (button ~= nil) then
			disableButton(button, "Already completed!");
		end
	else
		AddTrackedAchievement(achievementId);
	end
end

local function removeFromTracker(achievementId)
	local isTracked = IsTrackedAchievement(achievementId);
	if (isTracked) then
		RemoveTrackedAchievement(achievementId);
	end
end

local function OnLeave(self)
	GameTooltip:Hide()
end

local function comma_value(n) -- credit http://richard.warburton.it
	local left,num,right = string.match(n,'^([^%d]*%d)(%d*)(.-)$')
	return left..(num:reverse():gsub('(%d%d%d)','%1,'):reverse())..right
end

-- Shortage
local function checkShortage(dID, classRolesAvailable)
	for i = 1, LFG_ROLE_NUM_SHORTAGE_TYPES do
		local eligible, tank, healer, damage, itemCount = GetLFGRoleShortageRewards(dID, i);
		local tankLocked, healerLocked, damageLocked = GetLFDRoleRestrictions(dID);
		tank = tank and classRolesAvailable.tank and not tankLocked;
		healer = healer and classRolesAvailable.heal and not healerLocked;
		damage = damage and classRolesAvailable.dps and not damageLocked;
		if(eligible and itemCount > 0 and (tank or healer or damage)) then
			local data = {dID = dID, tank = tank, heal = healer, dps = damage};
			return data;
		end
	end
	
	return nil;
end

local function createShortageList()
	shortageList = {};
	
	local canTank, canHealer, canDamage = C_LFGList.GetAvailableRoles()
	local classRolesAvailable = {tank = canTank, heal = canHealer, dps = canDamage};
	
	for i = 1, GetNumRFDungeons() do
		local dID = GetRFDungeonInfo(i);
		local shortage = checkShortage(dID, classRolesAvailable);
		table.insert(shortageList, shortage);
	end
	
	for i = 1, GetNumRandomDungeons() do
        local dID = GetLFGRandomDungeonInfo(i)
		local shortage = checkShortage(dID, classRolesAvailable);
		table.insert(shortageList, shortage);
    end
	
	for i = 1, #DLUHQ.Panels.LFR.ShortageContainer.Content.lines do
		DLUHQ.Panels.LFR.ShortageContainer.Content.lines[i].text:SetText("");
	end
end

-- UI functions
local uniquealyzer = 1;
local checkButtons = {};

local function createCheckbutton(parent, x_loc, y_loc, displayname, option)
	uniquealyzer = uniquealyzer + 1;
	
	local checkbutton = CreateFrame("CheckButton", "checkbutton_0" .. uniquealyzer, parent, "ChatConfigCheckButtonTemplate");
	checkbutton:ClearAllPoints()
	checkbutton:SetPoint("TOPLEFT", x_loc, y_loc);
	getglobal(checkbutton:GetName() .. 'Text'):SetText(displayname);
	
	-- Option checkbox
	if (option) then
		checkButtons[option] = checkbutton;
	end

	return checkbutton;
end

function DLU.createTextRoot(text, alignment, parent, x_loc, y_loc, fontType)
	local title = parent:CreateFontString(nil, "ARTWORK", fontType);
	title:SetPoint(alignment, parent, alignment, x_loc, y_loc);
	title:SetText(text);
	return title;
end

function DLU.createTextChild(text, parent, below, x_loc, y_loc, fontType)
	local textChildNode = parent:CreateFontString(nil, "ARTWORK", fontType);
	textChildNode:SetPoint("TOPLEFT", below, "BOTTOMLEFT", x_loc, y_loc);
	textChildNode:SetText(text);
	return textChildNode;
end

local function createButton(parent, text, width, height, tooltip, x_loc, y_loc)
	local button = CreateFrame("Button", nil, parent, "UIPanelButtonTemplate");
	button:SetPoint("TOPLEFT", parent, "BOTTOMLEFT", x_loc, y_loc);
    button:SetText(text);
    button.tooltipText = tooltip;
    button:SetWidth(width);
    button:SetHeight(height);
	return button;
end

local function disableButton(button, text)
	button:SetText(text);
	button:Disable();
end

local function updateHonorValue()
	local honorLeft, honorLevel = DLU.honorLeftValues();
	
	local valueText = honorLeft;
	
	DLUHQ.Panels.Character.PvPBar.statusBar:SetValue(UnitHonor("player"));
	DLUHQ.Panels.Character.PvPBar.text:SetText(valueText);
end

local function updateHonorUI()
	updateHonorValue();
end

local function updateQuestUI()
	local _, numQuests = C_QuestLog.GetNumQuestLogEntries();
	
	local text = format("%i/%i", numQuests, MAX_QUESTS);
	
	if (numQuests >= MAX_QUESTS) then
		text = DLU.colorizeString(text, "red");
	end
	
	DLUHQ.Panels.Character.Quests.Value:SetText(text);
end

local function updateXpValue()
	local nextLevel, maxLevel = DLU.XpLabelInfo();
	if (DLU.player.level < maxLevel) then
		local xpNeeded, xpRested, xpPercentageNeeded, xpRestedPercentage, xpDisabled, maxLevel = DLU.XpLeft();
		
		if (xpDisabled) then
			DLUHQ.Panels.Character.XpBar.Label:SetText("Xp disabled");
			DLUHQ.Panels.Character.XpBar:SetValue(0);
		else
			DLUHQ.Panels.Character.XpBar:SetValue(100 - xpPercentageNeeded);
			DLUHQ.Panels.Character.XpBar.Label:SetText(format("%s (%i%%)", xpNeeded, xpPercentageNeeded));
			
			if (xpRested ~= nil) then
				addonData:ChangeStatusBarColor(DLUHQ.Panels.Character.XpBar, {
					r = .25,
					g = .5,
					b = 1
				});
			else
				addonData:ChangeStatusBarColor(DLUHQ.Panels.Character.XpBar, {
					r = .75,
					g = 0,
					b = .75
				});
			end
		end
	else
		local apNeeded, apNeededPercent = DLU.ArtifactXpLeftValue();
		local apValue;
		
		if (apNeeded ~= nil) then
			apValue = "%s (%i%%)";
			DLUHQ.Panels.Character.XpBar:SetValue(100 - apNeededPercent);
		else
			apValue = "No artifact weapon detected...";
			DLUHQ.Panels.Character.XpBar:SetValue(0);
		end
		
		addonData:ChangeStatusBarColor(DLUHQ.Panels.Character.XpBar, {
			r = .901,
			g = .8,
			b =.601
		});
		
		DLUHQ.Panels.Character.XpBar.Label:SetText(format(apValue, apNeeded, apNeededPercent));
	end
end

local function updateXpLabel()
	local nextLevel, maxLevel = DLU.XpLabelInfo();
	
	local text = "";
	
	if (DLU.player.level < maxLevel) then
		text = addonData.L.XpNeededForLevelX .. ":";
		DLUHQ.Panels.Character.Xp.Label:SetText(format(text, nextLevel));
	else
		local currentRank, traitsWaitingForSpending, nextRank = DLU.ArtifactTraits();
		
		if (currentRank ~= nil) then
			text = addonData.L.ApNeededForLevelX;
			if (traitsWaitingForSpending ~= nil and traitsWaitingForSpending > 0) then
				if (traitsWaitingForSpending > 1) then
					text = text .. " (" .. addonData.L.ApUpgrades .. ")";
				else
					text = text .. " (" .. addonData.L.ApUpgrade ..")";
				end
			end
		end
		
		if (text ~= "") then
			text = text .. ":";
		end
		
		DLUHQ.Panels.Character.Xp.Label:SetText(format(text, nextRank, traitsWaitingForSpending));
	end
end

local function updateXpUI()
	updateXpValue();
	--updateXpLabel();
end

local function updateAzeriteXp()
	local azeriteInfo = DLU.getAzeriteArtifactInfo()
	
	DLUHQ.Panels.Character.AzeriteBar.Label:SetText(azeriteInfo.xpNextLevel);
	DLUHQ.Panels.Character.AzeriteBar:SetValue(azeriteInfo.xpPercentage);
end

local function updateAzeriteXpUI()
	updateAzeriteXp();
end

local function checkItemLevelUIDisplay()
	if (MIN_PLAYER_LEVEL_FOR_ITEM_LEVEL_DISPLAY < DLU.player.level) then
		DLUHQ.Panels.Character.ItemLevel:Hide();
	end
end

local function updateItemLevelUI()
	local ilvl = DLU.getItemLevel();
	DLUHQ.Panels.Character.ItemLevel.Value:SetText(ilvl);
end

local function updateCurrencies()
	for k, v in ipairs(DLUCharacterSettings.currencies) do
		local currencyInfo = C_CurrencyInfo.GetCurrencyInfo(DLUCharacterSettings.currencies[k]);
		if (currencyInfo ~= nil and currencyInfo.iconFileID ~= nil and currencyInfo.discovered) then
			local icon = DLU.iconToTextSized(currencyInfo.iconFileID, 16, 16);
			local maxReached = false;
			
			if (currencyInfo.quantityEarnedThisWeek == currencyInfo.maxWeeklyQuantity and currencyInfo.maxWeeklyQuantity > 0 or currencyInfo.quantity == currencyInfo.maxQuantity and currencyInfo.maxQuantity > 0) then
				maxReached = true;
			end
			
			local amount = currencyInfo.quantity;
			if (DLUSettings.numberFormat == "DarkruneDK") then
				amount = DLU.numberFormat(amount);
			end
			
			local currencyString = format("%s %s", icon, amount);
			if (maxReached) then
				currencyString = DLU.colorizeString(currencyString, "red");
			end
			
			DLUHQ.Panels.Character["Currency" .. k].Id = k;
			DLUHQ.Panels.Character["Currency" .. k].Text:SetText(currencyString);
			DLUHQ.Panels.Character["Currency" .. k]:SetWidth(DLUHQ.Panels.Character["Currency" .. k].Text:GetWidth());
			DLUHQ.Panels.Character["Currency" .. k].Name = currencyInfo.name;
			DLUHQ.Panels.Character["Currency" .. k].WeeklyMax = currencyInfo.maxWeeklyQuantity;
			DLUHQ.Panels.Character["Currency" .. k].TotalMax = currencyInfo.maxQuantity;
		else
			texturePath = [[Interface\AddOns\DarkruneDKLevelUp\images\x]];
			local icon = format("|T%s:16:16:0:0:64:64:4:60:4:60|t", texturePath);
			
			DLUHQ.Panels.Character["Currency" .. k].Id = k;
			DLUHQ.Panels.Character["Currency" .. k].Text:SetText(format("%s %s", icon, "?"));
			DLUHQ.Panels.Character["Currency" .. k]:SetWidth(DLUHQ.Panels.Character["Currency" .. k].Text:GetWidth());
			DLUHQ.Panels.Character["Currency" .. k].Name = addonData.L.NotFound;
			DLUHQ.Panels.Character["Currency" .. k].WeeklyMax = 0;
			DLUHQ.Panels.Character["Currency" .. k].TotalMax = 0;
		end
	end
end

local function changeXpFormat(self, arg1, arg2, checked)
	DLUSettings["numberFormat"] = arg1;
	-- Update the text; if we merely wanted it to display newValue, we would not need to do this
	if (not checked) then
		UIDropDownMenu_SetText(DLU_NumberFormatSetting, DLUSettings["numberFormat"] .. " format");
		updateXpUI();
		updateAzeriteXpUI();
		updateHonorUI();
		updateCurrencies();
	end
	-- Because this is called from a sub-menu, only that menu level is closed by default.
	CloseDropDownMenus();
end

local function SetUpAddonOptions()
	local panel = CreateFrame("Frame", "DLUOptionsPanel", InterfaceOptionsFramePanelContainer);
	-- Option panel name
	panel.name = DLU.addonName;
	-- Add info to panel
	
	local author = GetAddOnMetadata(DLU.addonName, "Author");
	local authorBtag = GetAddOnMetadata(DLU.addonName, "X-BattleTag");
	
	local authorTextString = format("|c%s%s|r (%s)", DLU.addonColor, author, authorBtag);
	
	local title = DLU.createTextRoot(DLU.addonName, "TOPLEFT", panel, 16, -16, "GameFontNormalLarge");
	local authorText = DLU.createTextChild(DLU.colorizeString(addonData.L.MadeBy, "red") .. ": " .. authorTextString, panel, title, 0, -8, "GameFontHighlightSmall");
	local versionText = DLU.createTextChild(DLU.colorizeString(addonData.L.Version, "red") .. ": ".. DLU.addonVersion, panel, authorText, 0, -8, "GameFontHighlightSmall");
	local descriptionText = DLU.createTextChild(DLU.colorizeString(addonData.L.Description, "red") .. ": " .. GetAddOnMetadata(DLU.addonName, "X-Notes"), panel, versionText, 0, -8, "GameFontHighlightSmall");
	local websiteText = DLU.createTextChild(DLU.colorizeString(addonData.L.Website, "red") .. ": " .. GetAddOnMetadata(DLU.addonName, "X-Website") .. " (it's on Danish)", panel, descriptionText, 0, -8, "GameFontHighlightSmall");
	-- Add help button
	local helpButton = createButton(panel, "Help", 80, 22, "Shows a list of commands, that you can use.", 10, 30);
	helpButton:SetScript("OnClick", function()
		SlashCmdList.DLU("help");
	end);
	-- Add DarkruneDK HQ button
	local wqButton = createButton(panel, "Open DarkruneDK HQ", 160, 22, "Opens up the DarkruneDK HQ UI.", 100, 30);
	wqButton:SetScript("OnClick", function()
		SlashCmdList.DLU("hq");
	end);
	-- Add panel to addon options
	InterfaceOptions_AddCategory(panel);
	
	-- SettingsPanel
	local settingsPanel = CreateFrame("Frame", "DLUSettings", UIParent);
	settingsPanel.name = "Global settings";
	settingsPanel.parent = panel.name;
	InterfaceOptions_AddCategory(settingsPanel);
	-- SettingsPanel text
	local settingsPanelTitle = DLU.createTextRoot(settingsPanel.name, "TOPLEFT", settingsPanel, 16, -16, "GameFontNormalLarge");
	local settingsPanelDescription = DLU.createTextChild(format("Here you can change %s settings.", DLU.addonNameColored), settingsPanel, settingsPanelTitle, 0, -8, "GameFontHighlightSmall");
	-- Setting buttons
	local checkX = -60;
	for t, s in pairs(DLUSettings) do
		local settingType = type(s);
		if (settingType == "table") then
			local checkBox = createCheckbutton(settingsPanel, 10, checkX, s.text, t);
			checkBox:SetChecked(s.value);
			checkBox:SetScript("OnClick", function()
				local settingText;
				
				if (checkBox:GetChecked()) then
					DLUSettings[t].value = true;
					settingText = DLU.colorizeString("enabled", "green");
				else
					DLUSettings[t].value = false;
					settingText = DLU.colorizeString("disabled", "red");
				end
				
				DLU.createMessage(format("%s option %s!", DLUSettings[t].text, settingText));
			end);
			
			checkX = checkX - 20;
		end
	end
	
	local items = {
		"Blizzard",
		"DarkruneDK"
	};
	
	local dropDown = CreateFrame("Frame", "DLU_NumberFormatSetting", settingsPanel, "UIDropDownMenuTemplate")
	dropDown:SetPoint("TOPLEFT", 0, checkX - 20);
	UIDropDownMenu_SetWidth(dropDown, 200);
	UIDropDownMenu_JustifyText(dropDown, "LEFT");
	UIDropDownMenu_SetText(dropDown, DLUSettings["numberFormat"] .. " format");

	-- Create and bind the initialization function to the dropdown menu
	UIDropDownMenu_Initialize(dropDown, function(self, level, menuList)
		local info = UIDropDownMenu_CreateInfo()
		for _, text in pairs(items) do
			info.text, info.arg1 = text, text;
			if (DLUSettings["numberFormat"] == text) then
				info.checked = true;
			else
				info.checked = false;
			end
		
			info.menuList = 0;
			info.func = changeXpFormat;
			UIDropDownMenu_AddButton(info);
		end
	end);
	
	-- Add reset button
	local resetButton = createButton(settingsPanel, "Reset", 100, 22, "Resets the account-wide options for " .. DLU.addonNameColored .. "." , 10, 30);
	resetButton:SetScript("OnClick", function()
		-- Reset Options
		DLU.resetAddonOptions();
		
		-- Reload wow client to ensure settings are saved
		ReloadUI();
	end);
	
	-- Character settings
	local characterSettingsPanel = CreateFrame("Frame", "DLUCharacterSettingsPanel", UIParent);
	characterSettingsPanel.name = "Character settings";
	characterSettingsPanel.parent = panel.name;
	InterfaceOptions_AddCategory(characterSettingsPanel);
	--- Text
	local characterSettingsPanelTitle = DLU.createTextRoot(characterSettingsPanel.name, "TOPLEFT", characterSettingsPanel, 16, -16, "GameFontNormalLarge");
	local characterSettingsPanelText = DLU.createTextChild("Here you can change character specific settings.", characterSettingsPanel, characterSettingsPanelTitle, 0, -8, "GameFontHighlightSmall");
	local checkX = -60;
	for t, s in pairs(DLUCharacterSettings) do
		local settingType = type(s);
		if (settingType == "table" and DLUCharacterSettings[t].text ~= nil) then
			local checkBox = createCheckbutton(characterSettingsPanel, 10, checkX, s.text, t);
			
			if (t == "questingMode") then
				checkBox.tooltip = "Needs to reload UI";
			end
			
			checkBox:SetChecked(s.value);
			checkBox:SetScript("OnClick", function()
				local settingText;
				
				if (checkBox:GetChecked()) then
					DLUCharacterSettings[t].value = true;
					
					settingText = DLU.colorizeString("enabled", "green");
				else
					DLUCharacterSettings[t].value = false;
					settingText = DLU.colorizeString("disabled", "red");
				end
				
				if (t == "questingMode") then
					ReloadUI();
				end
				
				DLU.createMessage(format("%s option %s!", DLUCharacterSettings[t].text, settingText));
			end);
			
			checkX = checkX - 20;
		end
	end
	
	-- Add reset button
	local resetButton = createButton(characterSettingsPanel, "Reset", 100, 22, "Resets character specific options for " .. DLU.addonNameColored .. "." , 10, 30);
	resetButton:SetScript("OnClick", function()
		-- Reset Options
		DLU.resetCharacterOptions();
		
		-- Reload wow client to ensure settings are saved
		ReloadUI();
	end);
	
	-- Artifact panel
	local artifactPanel = CreateFrame("Frame", "dluArtifactOptionPanel", UIParent);
	artifactPanel.name = "Artifact";
	artifactPanel.parent = panel.name;
	InterfaceOptions_AddCategory(artifactPanel);
	-- Artifact panel text
	local artiTitle = DLU.createTextRoot(artifactPanel.name, "TOPLEFT", artifactPanel, 16, -16, "GameFontNormalLarge");
	local artiDescription = DLU.createTextChild("This page will have some buttons to help you track artifact achievements.", artifactPanel, artiTitle, 0, -8, "GameFontHighlightSmall");
	-- Hidden artifact	
	local hiddenArtifactAchievementTable = {
		["Dungeons completed"] = 11152,
		["World Quests completed"] = 11153,
		["Honorable kills"] = 11154
	};
	
	local buttonX = 10;
	for t, q in pairs(hiddenArtifactAchievementTable) do
		local trackText = "Track " .. t;
		local untrackText = "Untrack " .. t;
		local button = createButton(artifactPanel, trackText, 200, 22, "Tracks " .. t .. " needed for hidden artifact weapon appearance.", buttonX, 30);
		if (achievementCompleted(q)) then
			disableButton(button, "Already completed!");
		elseif (IsTrackedAchievement(q)) then
			button:SetText(untrackText);
		end
		button:SetScript("OnClick", function()
			local isTracked = IsTrackedAchievement(q);
			if (isTracked) then
				button:SetText(trackText);
				removeFromTracker(q);
			else
				button:SetText(untrackText);
				addToTracker(q);
			end
		end);
		buttonX = buttonX + 200;
	end
	
	-- Secrets --
	local secretPanel = CreateFrame("Frame", "DLUSecretPanel", UIParent);
	secretPanel.name = "Secrets"
	secretPanel.parent = panel.name;
	InterfaceOptions_AddCategory(secretPanel);
	-- Buttons
	local secretTitle = DLU.createTextRoot(secretPanel.name, "TOPLEFT", secretPanel, 16, -16, "GameFontNormalLarge");
	local uunaTitle = DLU.createTextChild("Uuna", secretPanel, secretTitle, 0, -8, "GameFontHighlightSmall");
	local uunaButton = addonData:CreateResponsiveButton(secretPanel, uunaTitle, "Uuna part 1", "Check Uuna secret status", 0, -16);
	uunaButton:SetScript("OnClick", function() 
		local steps = {
			{name = "1. A new friend", value = 50098},
			{name = "2. A new friend", value = 50099},
			{name = "3. A new friend", value = 50100},
			{name = "4. A new friend", value = 50101},
			{name = "Bright lights", value = 50102},
			{name = "Wanna be friends?", value = 50103},
			{name = "Finding Nuu", value = 50104},
			{name = "Shooting stars", value = 50105},
			{name = "Flower crown", value = 50106},
			{name = "Uuna gets kidnapped", value = 50107},
			{name = "Getting marked", value = 50108},
			{name = "Shadow of a Tear", value = 50109}
		};
		
		DLU.tableStatusOrdered(steps);
	end);
	
	local uunaWorldTourButton = addonData:CreateResponsiveButton(secretPanel, uunaTitle, "Uuna part 2 (World Tour)", "Check Uuna world tour status", uunaButton:GetWidth(), -16);
	uunaWorldTourButton:SetScript("OnClick", function()
		local steps = {
			{name = "Gate of the Setting Sun", value = 50140},
			{name = "Nighthold", value = 50141},
			{name = "Krasus Landing, Dalaran (Legion)", value = 50142},
			{name = "Dragonblight", value = 50143},
			{name = "Mount Hyjal", value = 50144},
			{name = "Kun-Lai Summit", value = 50145},
			{name = "Blackrock Mountain", value = 50146},
			{name = "Temple of Karabor", value = 50147}
		};
		
		DLU.tableStatusOrdered(steps);
	end);
	
	local baalTitle = DLU.createTextChild(format("Baa'l (%s)", DLU.colorizeString("requires uuna part 1 & 2", "red")), secretPanel, uunaButton, 0, -8, "GameFontHighlightSmall");
	local baalButton = addonData:CreateResponsiveButton(secretPanel, baalTitle, "Secret", "Check Baa'l status", 0, -16);
	baalButton:SetScript("OnClick", function()
		local steps = {
			{name = "Heart of Darkness, Nazmir", value = 52819},
			{name = "Drak'thul's Hideout, Broken Shore", value = 52809},
			{name = "Hook Point, Boralus", value = 52810},
			{name = "Atal'dazar, Zuldazar", value = 52818},
			{name = "Gol Osigr, Drustvar", value = 52817},
			{name = "Tortaka Refuge, Vol'dun", value = 52816},
			{name = "The Great Sea, Stormsong Valley", value = 52815},
			{name = "The South Seas, Nazmir", value = 52814},
			{name = "Proudmoore Keep, Boralus", value = 52813},
			{name = "Western Watch, Drustvar", value = 52812},
			{name = "Kennings Lodge, Tiragarde Sound", value = 53632},
			{name = "Waterfall Cave, Tiragarde Sound", value = 53633},
			{name = "Seaweed Cave, Boralus", value = 53634},
			{name = "Fatigue Cave, off the coast of Vol'dun", value = 52827}
		};
		
		DLU.tableStatusOrdered(steps);
	end);
end

-- DLUHQ
local BD_DATA = {
	bg = [[Interface\Tooltips\UI-Tooltip-Background]],
	edge = [[Interface\Buttons\WHITE8X8]]
};

DLU_BD = {
	primary = { bgFile = BD_DATA.bg, tile = true, tileSize = 16, edgeFile = BD_DATA.edge, edgeSize = 1 },
	secondary = { bgFile = BD_DATA.bg, tile = false, tileSize = 4, edgeFile = BD_DATA.edge, edgeSize = .75 }
};

local function getBackdropAlpha(frame)
	local _, _, _, a = frame:GetBackdropColor();
	return tonumber(format("%.1f", a));
end

local function hightlight(button)
	local alpha = getBackdropAlpha(button);
	if (alpha ~= .5) then
		button:SetBackdropColor(1,1,1,.4);
	end
end
local function unHightlight(button)
	local alpha = getBackdropAlpha(button);
	if (alpha ~= .5) then
		button:SetBackdropColor(1,1,1,.2);
	end
end

local function ShowXpTooltip()
	GameTooltip:ClearLines();
	GameTooltip:SetOwner(DLUHQ.Panels.Character.XpBar, 'ANCHOR_CURSOR', 0, -4);
	
	local nextLevel, maxLevel = DLU.XpLabelInfo();
	--print(nextLevel, maxLevel);
	if (DLU.player.level < maxLevel) then
		GameTooltip:AddLine(addonData.L.XP);
		GameTooltip:AddLine(" ");
		GameTooltip:AddDoubleLine(addonData.L.NextLevel, nextLevel,1,1,1);
		local _, xpRested, _, xpRestedPercentage = DLU.XpLeft();
		if (xpRested ~= nil) then
			GameTooltip:AddDoubleLine(addonData.L.RestedXp, format("%s", DLU.itemColorString(3, xpRested)), 1,1,1);
		end
	else
		local artifactName, spendPower, currentRank = DLU.ArtifactInfo();
		if (artifactName ~= nil) then
			GameTooltip:AddLine(artifactName, .9, .8, .5);
			GameTooltip:AddLine(" ");
			GameTooltip:AddDoubleLine(addonData.L.CurrentRank, currentRank, 1,1,1);
		end
	end
	
	GameTooltip:Show();
end

local function ShowHonorTooltip(self)
	local r,g,b,a = self.statusBar:GetStatusBarColor();
	self.statusBar:SetStatusBarColor(r,g,b,.6);
	
	GameTooltip:ClearLines();
	GameTooltip:SetOwner(DLUHQ.Panels.Character.PvPBar, 'ANCHOR_CURSOR', 0, -4);
	
	local _, honorLevel, remainingHonor, honorKills = DLU.honorLeftValues();
	
	GameTooltip:AddLine(addonData.L.PvP);
	GameTooltip:AddLine(" ");
	
	GameTooltip:AddDoubleLine(addonData.L.HonorLevel, honorLevel, 1,1,1);
	GameTooltip:AddDoubleLine(addonData.L.RemainingHonor, remainingHonor, 1,1,1);
	GameTooltip:AddDoubleLine(HONORABLE_KILLS, honorKills, 1,1,1);
	
	local currentConquest, requiredConquest = PVPGetConquestLevelInfo();
	
	GameTooltip:AddDoubleLine("Conquest", currentConquest .. " / " .. requiredConquest, 1,1,1);
	
	GameTooltip:Show();
end

local function ShowAzeriteTooltip()
	GameTooltip:ClearLines();
	GameTooltip:SetOwner(DLUHQ.Panels.Character.AzeriteBar, 'ANCHOR_CURSOR', 0, -4);
	
	local azeriteInfo = DLU.getAzeriteArtifactInfo();
	
	GameTooltip:AddLine(azeriteInfo.name);
	GameTooltip:AddLine(" ");
	GameTooltip:AddDoubleLine(addonData.L.Level, azeriteInfo.level, 1,1,1);
	
	GameTooltip:Show();
end

local function ShowCurrencyTooltip(self)
	GameTooltip:ClearLines();
	GameTooltip:SetOwner(self, 'ANCHOR_CURSOR', 0, -4);
	
	if (self.Text:GetText() ~= nil) then
		GameTooltip:AddLine(self.Name);
		
		if (self.WeeklyMax > 0 or self.TotalMax > 0) then
			GameTooltip:AddLine(" ");
			
			if (self.WeeklyMax > 0) then
				GameTooltip:AddDoubleLine("Weekly", self.WeeklyMax,1,1,1);
			end
			if (self.TotalMax > 0) then
				GameTooltip:AddDoubleLine("Max", self.TotalMax,1,1,1);
			end
		end
		
		GameTooltip:Show();
	end
end

local function ShowSimpleTooltip(self)
	GameTooltip:ClearLines();
	GameTooltip:SetOwner(self, 'ANCHOR_CURSOR', 0, -4);
	
	if (self.Tooltip ~= nil) then
		GameTooltip:AddLine(self.Tooltip);
		GameTooltip:Show();
	end
end

local function HideTooltip(self)
	if (self.statusBar ~= nil) then
		local r,g,b,a = self.statusBar:GetStatusBarColor();
		self.statusBar:SetStatusBarColor(r,g,b,1);
	end
	
	GameTooltip:Hide();
end

local function UpdateCurrency(self)
	DLUCharacterSettings.currencies[editCurrencyId] = self.CurrencyId;
	editCurrencyId = nil;
	
	updateCurrencies();
	DLUCurrencySelector:Hide();
	DLUCurrencySelector.backButton:Hide();
end

function addonData:CreateLabel(text, fontSize, alignment, parentFrame)
	if (parentFrame == nil) then
		return
	end

	local label = parentFrame:CreateFontString();
	label:SetFont("Fonts\\FRIZQT__.TTF", fontSize);
	label:SetPoint(alignment, parentFrame, alignment, 0, 0);
	label:SetJustifyH("LEFT");
    label:SetJustifyV("TOP");
    label:SetText(text);
    label:SetTextColor(1, 1, 1, 1);
    return label;
end

function addonData:CreateButton(width, height, alignment, parentFrame, xOffset, yOffset)
	if (parentFrame == nil) then
		return
	end

	xOffset = xOffset or 0;
	yOffset = yOffset or 0;

	local button = CreateFrame("Button", nil, parentFrame, BackdropTemplateMixin and "BackdropTemplate");
	button:SetPoint(alignment, parentFrame, alignment, xOffset, yOffset);
	button:SetSize(width, height);
	button:SetBackdrop(DLU_BD.secondary);
	button:SetBackdropColor(1,1,1,.2);
	button:SetBackdropBorderColor(1,1,1,0);
	button:SetScript("OnEnter", hightlight);
	button:SetScript("OnLeave", unHightlight);
	return button;
end

function addonData:CreatePanel(width, height, parentFrame, yOffset)
	local frame = CreateFrame("Frame", nil, parentFrame, BackdropTemplateMixin and "BackdropTemplate");
	frame:SetSize(width, height);
	frame:SetPoint("TOPLEFT", parentFrame, "TOPLEFT", 0, yOffset);
	frame:SetBackdrop(DLU_BD.secondary);
	frame:SetBackdropColor(0,0,0,.9);
	frame:SetBackdropBorderColor(1,1,1,0);
	return frame;
end

function addonData:CreateStatusBar(parent, width, height, position, xOffset, yOffset, color, hover)
	local frame = CreateFrame("StatusBar", nil, parent);
	frame:SetSize(width, height);
	frame:SetPoint(position, parent, position, xOffset, yOffset);
	frame:SetStatusBarTexture([[Interface\BUTTONS\WHITE8X8.blp]]);
	frame:SetStatusBarColor(color.r, color.g, color.b, color.a);
	frame:SetMinMaxValues(0,100);
	frame:SetScript("OnEnter", hover);
	frame:SetScript("OnLeave", HideTooltip);
	
	return frame;
end

function addonData:ChangeStatusBarColor(statusbar, newColor)
	statusbar:SetStatusBarColor(newColor.r, newColor.g, newColor.b, .6);
end

function addonData:CreateBorder(parent)
	local frame = CreateFrame("Frame", nil, parent, BackdropTemplateMixin and "BackdropTemplate");
	frame:SetPoint("TOPLEFT", parent, "TOPLEFT", -.5, .5);
	frame:SetPoint("BOTTOMRIGHT", parent, "BOTTOMRIGHT", .5, -.5);
	frame:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = false, tileSize = 0, edgeSize = 1, 
											insets = { left = -1, right = -1, top = -1, bottom = -1}
                                            });
	frame:SetBackdropColor(0,0,0,.6);
	frame:SetFrameLevel(parent:GetFrameLevel());
	return frame;
end

function addonData:CreateDLUButton(Parent, OnEnter, OnLeave, OnClick, width, height, position, xOffset, yOffset)
	local button = CreateFrame("Button", nil, Parent);
	button:SetPoint(position, Parent, position, xOffset, yOffset);
	button:SetSize(width, height);
	button.Text = addonData:CreateLabel("", FONT_SIZES.medium, "LEFT", button);
	button:SetScript("OnEnter", OnEnter);
	button:SetScript("OnLeave", OnLeave);
	
	if (OnClick ~= nil) then
		button:SetScript("OnClick", OnClick);
	end
	
	return button;
end

function addonData:CreateBar(Parent, OnEnter, OnClick, width, height, position, xOffset, yOffset, color)
	-- Button
	local bar = CreateFrame("Button", nil, Parent, BackdropTemplateMixin and "BackdropTemplate");
	bar:SetPoint(position, Parent, position, xOffset, yOffset);
	bar:SetSize(width, height);
	bar:SetScript("OnEnter", OnEnter);
	bar:SetScript("OnLeave", HideTooltip);
	if (OnClick ~= nil) then
		bar:SetScript("OnClick", OnClick);
	end
	
	bar:SetBackdrop({bgFile = "Interface/Tooltips/UI-Tooltip-Background", 
                                            edgeFile = "Interface/Tooltips/UI-Tooltip-Border", 
                                            tile = false, tileSize = 0, edgeSize = 1, 
											insets = { left = -1, right = -1, top = -1, bottom = -1}
                                            });
	bar:SetBackdropColor(0,0,0,.6);
	
	-- Status bar
	bar.statusBar = CreateFrame("StatusBar", nil, bar);
	
	bar.statusBar:SetPoint('TOPLEFT', bar, 'TOPLEFT', 0, 0);
	bar.statusBar:SetPoint('BOTTOMRIGHT', bar, 'BOTTOMRIGHT', 0, 0);
	
	bar.statusBar:SetStatusBarTexture([[Interface\BUTTONS\WHITE8X8.blp]]);
	bar.statusBar:SetStatusBarColor(color.r, color.g, color.b, color.a);
	bar.statusBar:SetMinMaxValues(0,100);
	
	-- Text
	bar.text = addonData:CreateLabel("", FONT_SIZES.small, "CENTER", bar.statusBar);
	
	return bar;
end

function addonData:CreateResponsiveButton(parent, relativeTo, text, tooltip, x_loc, y_loc)
	local button = CreateFrame("Button", nil, parent, "UIPanelButtonTemplate");
	button:SetPoint("TOPLEFT", relativeTo, "TOPLEFT", x_loc, y_loc);
    button:SetText(text);
	if (tooltip ~= "") then
		button.tooltipText = tooltip;
	end
    button:SetWidth(button.Text:GetWidth() + 40);
    button:SetHeight(button.Text:GetHeight() + 10);
	return button;
end

local function toggleContainer()
	local isVisible = DLUHQ.Container:IsVisible();
	local name = DLUHQ:GetName();
	
	if (not DLU_UI[name]) then
		DLU_UI[name] = {};
	end
	
	if (isVisible) then
		DLUHQ.Container:Hide();
		DLUHQ.minimize.Label:SetText("+");
	else
		DLUHQ.Container:Show();
		DLUHQ.minimize.Label:SetText("-");
	end
	
	DLU_UI[name].minimized = isVisible;
end

local function scroll(self, sv)
	sv = sv * 16;
	local maximum = max(0, DLUHQ.Panels.LFR.ShortageContainer.Content:GetHeight() - self:GetHeight());
	local value = self:GetVerticalScroll();
	if ((value - sv) < 0) then
		self:SetVerticalScroll(0);
	elseif (value - sv) > maximum then
		self:SetVerticalScroll(maximum)
	else
		self:SetVerticalScroll(value - sv)
	end
end

local function stopUIMoving(self)
	self:StopMovingOrSizing();
	local name = self:GetName();
	
	if (DLU_UI[name]) then
		DLU_UI[name]["xPos"] = self:GetLeft();
		DLU_UI[name]["yPos"] = self:GetBottom();
	else
		DLU_UI[name] = {};
		DLU_UI[name].xPos = self:GetLeft();
		DLU_UI[name].yPos = self:GetBottom();
	end
end

-- Navigation
local function togglePanel(shownPanel)
	for k, v in pairs(DLUHQ.Panels) do
		if (k == shownPanel) then		
			DLUHQ[shownPanel]:SetBackdropColor(1,1,1,.5);
			DLUHQ.Panels[shownPanel]:Show();
			DLU_UI["shownPanel"] = shownPanel;
		else
			DLUHQ[k]:SetBackdropColor(1,1,1,.2);
			DLUHQ.Panels[k]:Hide();
		end
	end
end

local function Honor_OnClick()
	PVEFrame_ShowFrame("PVPUIFrame");
end

local expansions = {EXPANSION_NAME0, EXPANSION_NAME1, EXPANSION_NAME2, EXPANSION_NAME3, EXPANSION_NAME4, EXPANSION_NAME5, EXPANSION_NAME6, EXPANSION_NAME7, EXPANSION_NAME8};
local expansionIcons = {686, 697, 4532, 6116, 8679, 10019, 12002, 14068, 14418};
local currencies = {
	{515,1379,81,1166},
	{1704},
	{241,61},
	{416,615,614,361,391},
	{697,738,777,752,789,776,402},
	{823,824,1101,994,1129,980,944},
	{1275,1533,1508,1226,1149,1416,1273,1155,1220,1342,1314,1356,1357,1268,1355,1154},
	{1560,1716,1717,1580,1721,1710,1755,1719,1718,1803},
	{1754} -- 1792,1820,1751,1802,1816,1831,1830,1832,1829
}

local function HideCurrencyContainers()
	for i = 1, #DLUCurrencySelector.currencyContainer do
		DLUCurrencySelector.currencyContainer[i]:Hide();
	end
end

local function ShowAvailableCurrencies(selectedExpansion)
	DLUCurrencySelector.expansionContainer:Hide();
	DLUCurrencySelector.backButton:Show();
	HideCurrencyContainers();
	DLUCurrencySelector.currencyContainer[selectedExpansion]:Show();
end

local function ShowCurrencySelectorUI(self)
	DLUCurrencySelector:Show();
	DLUCurrencySelector.expansionContainer:Show();
	HideCurrencyContainers();
	
	editCurrencyId = self.Id;
end

local function CreateCurrencySelectorUI()
	local frame = CreateFrame("Frame", "DLUCurrencySelector", UIParent, BackdropTemplateMixin and "BackdropTemplate");
	frame:SetSize(200, 120);
	frame:SetPoint("CENTER", UIParent, "CENTER");
	frame:SetBackdrop(DLU_BD.secondary);
	frame:SetBackdropColor(0,0,0,1);
	frame:SetBackdropBorderColor(1,1,1,0);
	
	-- Close button
	frame.close = addonData:CreateButton(16, 16, "TOPRIGHT", frame);
	frame.close.Label = addonData:CreateLabel("X", 10, "CENTER", frame.close);
	frame.close:SetScript("OnClick", function()
		frame:Hide();
		frame.backButton:Hide();
	end);
	
	frame.backButton = addonData:CreateButton(16, 16, "TOPLEFT", frame);
	frame.backButton:SetBackdrop(nil);
	local _, _, backIcon = GetSpellInfo(131128);
	backIcon = format("|T%s:16:16:0:0:64:64:4:60:4:60|t", backIcon);
	
	frame.backButton.Label = addonData:CreateLabel(backIcon, 10, "CENTER", frame.backButton);
	frame.backButton.Tooltip = "Go back";
	frame.backButton:SetScript("OnClick", function(self)
		HideCurrencyContainers();
		frame.expansionContainer:Show();
		self:Hide();
	end);
	frame.backButton:SetScript("OnEnter", ShowSimpleTooltip);
	frame.backButton:SetScript("OnLeave", HideTooltip);
	frame.backButton:Hide();
	
	-- Title
	frame.title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
	frame.title:SetPoint("TOP", 0, -2);
	frame.title:SetText(CURRENCY .. " selector");
	
	-- expansions
	frame.expansionContainer = CreateFrame("Frame", nil, frame);
	frame.expansionContainer:SetSize(200, 80);
	frame.expansionContainer:SetPoint("BOTTOM", frame, "BOTTOM", 0, 0);
	
	frame.expansionContainer.button = {};
	frame.currencyContainer = {};
	
	local counter = 0;
	local yCoord = 0;
	
	local currencyYCoord = 0;
	local currencyCounter = 0;
	
	for i = 1, GetNumExpansions() do
		local xCoord = 2;
		if (i % 7 == 0) then
			yCoord = yCoord + 34;
			counter = 0;
		end
		
		xCoord = 34 * counter;
		
		frame.expansionContainer.button[i] = addonData:CreateButton(32, 32, "TOPLEFT", frame.expansionContainer, xCoord, -yCoord);
		
		local _, _, _, _, _, _, _, _, _, icon = GetAchievementInfo(expansionIcons[i]);
		
		frame.expansionContainer.button[i].icon = frame.expansionContainer.button[i]:CreateTexture(nil, "BACKGROUND", nil, -8);
		frame.expansionContainer.button[i].icon:SetAllPoints();
		frame.expansionContainer.button[i].icon:SetTexture(icon);
		frame.expansionContainer.button[i].Tooltip = expansions[i];
		
		frame.expansionContainer.button[i]:SetScript("OnEnter", ShowSimpleTooltip);
		frame.expansionContainer.button[i]:SetScript("OnLeave", HideTooltip);
		frame.expansionContainer.button[i]:SetScript("OnClick", function()
			ShowAvailableCurrencies(i);
		end);
		
		-- Currency container
		frame.currencyContainer[i] = CreateFrame("Frame", nil, frame);
		frame.currencyContainer[i]:SetSize(200, 80);
		frame.currencyContainer[i]:SetPoint("BOTTOM", frame, "BOTTOM", 0, 0);
		frame.currencyContainer[i]:Hide();
		frame.currencyContainer[i].currency = {};
		
		local availableCurrencies = currencies[i];
		
		for y = 1, #availableCurrencies do
			local currencyXCoord = 2;
			if (y % 12 == 0) then
				currencyYCoord = currencyYCoord + 18;
				currencyCounter = 0;
			end
			
			currencyXCoord = 18 * currencyCounter;
			
			local currencyInfo = C_CurrencyInfo.GetCurrencyInfo(availableCurrencies[y]);
			local icon = format("|T%s:16:16:0:0:64:64:4:60:4:60|t", currencyInfo.iconFileID);
			
			frame.currencyContainer[i].currency[y] = addonData:CreateButton(16, 16, "TOPLEFT", frame.currencyContainer[i], currencyXCoord, -currencyYCoord);
			frame.currencyContainer[i].currency[y]:SetBackdrop(nil);
			frame.currencyContainer[i].currency[y].Label = addonData:CreateLabel(icon, 10, "CENTER", frame.currencyContainer[i].currency[y]);
			frame.currencyContainer[i].currency[y].Tooltip = currencyInfo.name;
			frame.currencyContainer[i].currency[y].CurrencyId = availableCurrencies[y];
			frame.currencyContainer[i].currency[y]:SetScript("OnClick", UpdateCurrency);
			frame.currencyContainer[i].currency[y]:SetScript("OnEnter", ShowSimpleTooltip);
			frame.currencyContainer[i].currency[y]:SetScript("OnLeave", HideTooltip);
			
			if (y == #availableCurrencies) then
				currencyCounter = 0;
				currencyYCoord = 0;
			else
				currencyCounter = currencyCounter + 1;
			end
		end
		
		counter = counter + 1;
	end
	
	frame:Hide();
end

local function selectTalentTemplate(self)
	if (DLU.CanChangeTemplate()) then
		UIDropDownMenu_SetSelectedID(DLUTalentDropDown, self:GetID());
		DLU.LoadTemplate(self:GetText());
		selectedTalentTemplate = self:GetText();
	else
		DLU.createMessage(addonData.L.TalentChangeError, "error");
	end
end

local function CreateTalentUI()
	local frame = CreateFrame("Frame", "DLUTalentFrame", UIParent, BackdropTemplateMixin and "BackdropTemplate");
	frame:SetSize(200, 120);
	frame:SetPoint("CENTER", UIParent, "CENTER");
	frame:SetBackdrop(DLU_BD.secondary);
	frame:SetBackdropColor(0,0,0,1);
	frame:SetBackdropBorderColor(1,1,1,0);
	
	-- Close button
	frame.close = addonData:CreateButton(16, 16, "TOPRIGHT", frame);
	frame.close.Label = addonData:CreateLabel("X", 10, "CENTER", frame.close);
	frame.close:SetScript("OnClick", function()
		frame:Hide();
	end);
	
	-- Title
	frame.title = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
	frame.title:SetPoint("TOP", 0, -2);
	frame.title:SetText(addonData.L.TalentManager);
	
	-- Talent dropdown
	local dropDown = CreateFrame("Frame", "DLUTalentDropDown", frame, "UIDropDownMenuTemplate")
	dropDown:SetPoint("TOPLEFT", 0, -22);
	UIDropDownMenu_SetWidth(dropDown, 150);
	UIDropDownMenu_JustifyText(dropDown, "LEFT");
	UIDropDownMenu_SetText(dropDown, addonData.L.TalentDropdownDefault);

	-- Create and bind the initialization function to the dropdown menu
	UIDropDownMenu_Initialize(dropDown, function(self, level, menuList)
		local info = UIDropDownMenu_CreateInfo()
		
		if (DLUCharacterSettings.Talents == nil) then
			DLUCharacterSettings.Talents = {};
		end
		
		if (#DLUCharacterSettings.Talents > 0) then
			for i = 1, #DLUCharacterSettings.Talents do
				info = UIDropDownMenu_CreateInfo();
				info.text = DLUCharacterSettings.Talents[i].name;
				info.value = DLUCharacterSettings.Talents[i].name;
				info.func = selectTalentTemplate;
				UIDropDownMenu_AddButton(info, level);
			end
		else
			UIDropDownMenu_SetText(dropDown, addonData.L.TalentDropdownNoFound);
		end
	end);
	
	local editBox = CreateFrame("EditBox", "DLUTemplateEditBox", frame, "InputBoxTemplate");
	editBox:SetSize(180, 24);
	editBox:SetMultiLine(false);
	editBox:SetAutoFocus(false);
	editBox:SetMaxLetters(24);
	editBox:SetPoint("TOPLEFT", 14, -50);
	
	local addButton = addonData:CreateButton(16, 16, "BOTTOMLEFT", frame, 0, 0);
	addButton.Label = addonData:CreateLabel(addonData.L.Save, 10, "CENTER", addButton);
	addButton:SetWidth(addButton.Label:GetWidth() + 4);
	addButton:SetScript("OnClick", function()
		local templateName = editBox:GetText();
		DLU.SaveTemplate(templateName);
		editBox:SetText("");
	end);
	
	local removeButton = addonData:CreateButton(16, 16, "BOTTOMRIGHT", frame, 0, 0);
	removeButton.Label = addonData:CreateLabel(addonData.L.Delete, 10, "CENTER", removeButton);
	removeButton:SetWidth(removeButton.Label:GetWidth() + 4);
	removeButton:SetScript("OnClick", function()
		if (selectedTalentTemplate ~= nil) then
			DLU.DeleteTemplate(selectedTalentTemplate, true);
		end
	end);
	
	frame:Hide();
end

local function createDLUHQUI()
	DLUHQ = CreateFrame("Frame", "DLUHQ_UI", UIParent, BackdropTemplateMixin and "BackdropTemplate");
	DLUHQ:SetSize(WIDTH, 20);
	DLUHQ:SetBackdrop(DLU_BD.primary);
	
	local name = DLUHQ:GetName();
	if (DLU_UI[name] and DLU_UI[name].xPos) then
		DLUHQ:SetPoint("BOTTOMLEFT", DLU_UI[name].xPos, DLU_UI[name].yPos);
	else
		DLUHQ:SetPoint("CENTER", UIParent, "CENTER", 0, (HEIGHT + 20 + 25) / 2);
	end
	
	--DLUHQ:SetFrameStrata("DIALOG");
	DLUHQ:SetBackdropColor(0,0,0,1);
	DLUHQ:SetBackdropBorderColor(.7,.7,.7,1);
	DLUHQ:SetClampedToScreen(true);
	DLUHQ:SetMovable(true);
    DLUHQ:EnableMouse(true);
	DLUHQ:RegisterForDrag("LeftButton");

	DLUHQ:SetScript("OnDragStart", DLUHQ.StartMoving);
    DLUHQ:SetScript("OnDragStop", stopUIMoving);

	-- Title
	DLUHQ.title = DLUHQ:CreateFontString(nil, "ARTWORK", "GameFontNormalLarge");
	DLUHQ.title:SetPoint("TOP", 0, -2);
	DLUHQ.title:SetText(format("%s HQ", "DarkruneDK"));

	-- Settings button
	DLUHQ.settings = addonData:CreateButton(16, 16, "LEFT", DLUHQ);
	DLUHQ.settings.Label = addonData:CreateLabel("?", 10, "CENTER", DLUHQ.settings);
	DLUHQ.settings:SetScript("OnClick", function()
		InterfaceOptionsFrame_OpenToCategory(addonName);
	end);

	-- Minimize button
	DLUHQ.minimize = addonData:CreateButton(16, 16, "RIGHT", DLUHQ, -20);
	DLUHQ.minimize.Label = addonData:CreateLabel("-", 10, "CENTER", DLUHQ.minimize);
	DLUHQ.minimize:SetScript("OnClick", toggleContainer);

	-- Close button
	DLUHQ.close = addonData:CreateButton(16, 16, "RIGHT", DLUHQ);
	DLUHQ.close.Label = addonData:CreateLabel("X", 10, "CENTER", DLUHQ.close);
	DLUHQ.close:SetScript("OnClick", function()
		DLUHQ:Hide();
	end);

	-- Panel Container
	DLUHQ.Container = CreateFrame("Frame", nil, DLUHQ);
	DLUHQ.Container:SetSize(WIDTH, HEIGHT);
	DLUHQ.Container:SetPoint("TOPLEFT", DLUHQ, "TOPLEFT", 0, -20);
	if (DLU_UI[name] and DLU_UI[name].minimized) then
		toggleContainer();
	end

	-- Panel Navigation
	DLUHQ.navigation = CreateFrame("Frame", nil, DLUHQ.Container, BackdropTemplateMixin and "BackdropTemplate");
	DLUHQ.navigation:SetSize(WIDTH, HEIGHT_NAV);
	DLUHQ.navigation:SetPoint("TOPLEFT", DLUHQ.Container, "TOPLEFT", 0, 0);
	DLUHQ.navigation:SetBackdrop(DLU_BD.secondary);
	DLUHQ.navigation:SetBackdropColor(.7,0,0, 1);
	DLUHQ.navigation:SetBackdropBorderColor(1,1,1,0);

	-- Panel Buttons
	DLUHQ.Character = addonData:CreateButton(75, HEIGHT_NAV, "TOPLEFT", DLUHQ.navigation)
	DLUHQ.Character.Label = addonData:CreateLabel(addonData.L.Character, FONT_SIZES.small, "CENTER", DLUHQ.Character);

	DLUHQ.LFR = addonData:CreateButton(75, HEIGHT_NAV, "TOPLEFT", DLUHQ.navigation, 75 + 3);
	DLUHQ.LFR.Label = addonData:CreateLabel(addonData.L.LFR, FONT_SIZES.small, "CENTER", DLUHQ.LFR);
	
	DLUHQ.Tips = addonData:CreateButton(75, HEIGHT_NAV, "TOPLEFT", DLUHQ.navigation, (75+3)*2);
	DLUHQ.Tips.Label = addonData:CreateLabel(addonData.L.Tips, FONT_SIZES.small, "CENTER", DLUHQ.Tips);

	------------
	-- Panels --
	------------
	DLUHQ.Panels = {};
	
	--- Character
	DLUHQ.Panels.Character = addonData:CreatePanel(WIDTH, HEIGHT - HEIGHT_NAV, DLUHQ.Container, -HEIGHT_NAV);
	DLU.createTextRoot(addonData.L.Character, "TOP", DLUHQ.Panels.Character, 8, -6, "GameFontNormalLarge");
	
	-- New XP/AP (status bar)
	DLUHQ.Panels.Character.XpBar = addonData:CreateStatusBar(DLUHQ.Panels.Character, WIDTH - PADDING, FONT_SIZES.medium, "TOPLEFT", PADDING/2, -(5 + PADDING_TOP), {
		r = 1,
		g = .2,
		b = .8
	}, ShowXpTooltip);
	
	DLUHQ.Panels.Character.XpBar.border = addonData:CreateBorder(DLUHQ.Panels.Character.XpBar);
	DLUHQ.Panels.Character.XpBar.Label = addonData:CreateLabel("", FONT_SIZES.medium-2, "CENTER", DLUHQ.Panels.Character.XpBar);
	updateXpUI();
	
	-- PvP/Honor
	DLUHQ.Panels.Character.PvPBar = addonData:CreateBar(DLUHQ.Panels.Character, ShowHonorTooltip, Honor_OnClick, WIDTH - PADDING, FONT_SIZES.medium + 2, "TOPLEFT", PADDING/2, -(5 + PADDING_TOP + FONT_SIZES.medium + 6), {
		r = .941,
		g = .552,
		b = .254
	});
	DLUHQ.Panels.Character.PvPBar.statusBar:SetMinMaxValues(0, UnitHonorMax("player"));
	updateHonorUI();
	
	-- Quests
	DLUHQ.Panels.Character.Quests = CreateFrame("Frame", nil, DLUHQ.Panels.Character);
	DLUHQ.Panels.Character.Quests:SetSize(WIDTH - PADDING, FONT_SIZES.medium);
	DLUHQ.Panels.Character.Quests:SetPoint("TOPLEFT", DLUHQ.Panels.Character, "TOPLEFT", PADDING/2, -(5 + PADDING_TOP + FONT_SIZES.medium + 12 * 2))
	
	DLUHQ.Panels.Character.Quests.Label = addonData:CreateLabel("Amount of Quests:", FONT_SIZES.medium, "LEFT", DLUHQ.Panels.Character.Quests);
	DLUHQ.Panels.Character.Quests.Value = addonData:CreateLabel("", FONT_SIZES.medium, "RIGHT", DLUHQ.Panels.Character.Quests);
	updateQuestUI();
	
	-- Item level
	DLUHQ.Panels.Character.ItemLevel = CreateFrame("Frame", nil, DLUHQ.Panels.Character);
	DLUHQ.Panels.Character.ItemLevel:SetSize(WIDTH - PADDING, FONT_SIZES.medium);
	DLUHQ.Panels.Character.ItemLevel:SetPoint("TOPLEFT", DLUHQ.Panels.Character, "TOPLEFT", PADDING/2, -(5 + PADDING_TOP + FONT_SIZES.medium + 12 * 3));
	
	DLUHQ.Panels.Character.ItemLevel.Label = addonData:CreateLabel("Item level:", FONT_SIZES.medium, "LEFT", DLUHQ.Panels.Character.ItemLevel);
	DLUHQ.Panels.Character.ItemLevel.Value = addonData:CreateLabel("0", FONT_SIZES.medium, "RIGHT", DLUHQ.Panels.Character.ItemLevel);
	checkItemLevelUIDisplay();
	if (DLUHQ.Panels.Character.ItemLevel:IsVisible()) then
		updateItemLevelUI();
	end
	
	-- Azerite
	DLUHQ.Panels.Character.AzeriteBar = addonData:CreateStatusBar(DLUHQ.Panels.Character, WIDTH - PADDING, FONT_SIZES.medium, "TOPLEFT", PADDING/2, -((5 + PADDING_TOP + FONT_SIZES.medium + 12 * 5) - 6), {
		r = .901,
		g = .8,
		b =.601
	}, ShowAzeriteTooltip);
	DLUHQ.Panels.Character.AzeriteBar.border = addonData:CreateBorder(DLUHQ.Panels.Character.AzeriteBar);
	DLUHQ.Panels.Character.AzeriteBar.Label = addonData:CreateLabel("TESTING!", FONT_SIZES.medium-2, "CENTER", DLUHQ.Panels.Character.AzeriteBar);
	DLUHQ.Panels.Character.AzeriteBar:SetMinMaxValues(0, 100);
	
	local azeriteInfo = DLU.getAzeriteArtifactInfo();
	if (azeriteInfo.isEquipped ~= true) then
		DLUHQ.Panels.Character.AzeriteBar:Hide();
	else 
		updateAzeriteXpUI();
	end
	
	-- Currency
	DLUHQ.Panels.Character.Currency1 = addonData:CreateDLUButton(DLUHQ.Panels.Character, ShowCurrencyTooltip, HideTooltip, ShowCurrencySelectorUI, 30, 30, "BOTTOMLEFT", 8, 0);
	DLUHQ.Panels.Character.Currency2 = addonData:CreateDLUButton(DLUHQ.Panels.Character, ShowCurrencyTooltip, HideTooltip, ShowCurrencySelectorUI, 30, 30, "BOTTOMLEFT", WIDTH_FIFTH+8, 0);
	DLUHQ.Panels.Character.Currency3 = addonData:CreateDLUButton(DLUHQ.Panels.Character, ShowCurrencyTooltip, HideTooltip, ShowCurrencySelectorUI, 30, 30, "BOTTOMLEFT", WIDTH_FIFTH*2+8, 0);
	DLUHQ.Panels.Character.Currency4 = addonData:CreateDLUButton(DLUHQ.Panels.Character, ShowCurrencyTooltip, HideTooltip, ShowCurrencySelectorUI, 30, 30, "BOTTOMLEFT", WIDTH_FIFTH*3+8, 0);
	DLUHQ.Panels.Character.Currency5 = addonData:CreateDLUButton(DLUHQ.Panels.Character, ShowCurrencyTooltip, HideTooltip, ShowCurrencySelectorUI, 30, 30, "BOTTOMLEFT", WIDTH_FIFTH*4+8, 0);
	
	updateCurrencies();

	--- LFR
	DLUHQ.Panels.LFR = addonData:CreatePanel(WIDTH, HEIGHT - HEIGHT_NAV, DLUHQ.Container, -HEIGHT_NAV);
	DLU.createTextRoot(addonData.L.LFR, "TOP", DLUHQ.Panels.LFR, 8, -6, "GameFontNormalLarge");
	DLUHQ.Panels.LFR.ShortageContainer = CreateFrame("ScrollFrame", nil, DLUHQ.Panels.LFR);
	DLUHQ.Panels.LFR.ShortageContainer.Content = CreateFrame("Frame", nil, DLUHQ.Panels.LFR.ShortageContainer);
	DLUHQ.Panels.LFR.ShortageContainer:SetScrollChild(DLUHQ.Panels.LFR.ShortageContainer.Content);
	DLUHQ.Panels.LFR.ShortageContainer:SetScript("OnMouseWheel", scroll);
	DLUHQ.Panels.LFR.ShortageContainer:SetSize(WIDTH, 300);
	DLUHQ.Panels.LFR.ShortageContainer:SetPoint("TOPLEFT", DLUHQ.Panels.LFR, "TOPLEFT", 0, -PADDING_TOP);

	DLUHQ.Panels.LFR.ShortageContainer.Content:SetSize(WIDTH, 300);
	DLUHQ.Panels.LFR.ShortageContainer.Content.lines = {};
	for i = 1, GetNumRFDungeons() do
		local line = CreateFrame("Frame", nil, DLUHQ.Panels.LFR.ShortageContainer.Content);
		line:SetPoint("TOPLEFT", 0, -(i-1)*16);
		line:SetSize(1, 16);

		line.text = line:CreateFontString(nil, "ARTWORK", "GameFontWhite");
		line.text:SetPoint("LEFT", 5, -5);

		DLUHQ.Panels.LFR.ShortageContainer.Content.lines[i] = line;
	end
	
	--- Tips
	DLUHQ.Panels.Tips = addonData:CreatePanel(WIDTH, HEIGHT - HEIGHT_NAV, DLUHQ.Container, -HEIGHT_NAV);
	DLU.createTextRoot(addonData.L.Tips, "TOP", DLUHQ.Panels.Tips, 8, -6, "GameFontNormalLarge");
	
	DLUHQ.Panels.Tips.Container = CreateFrame("Frame", nil, DLUHQ.Panels.Tips);
	DLUHQ.Panels.Tips.Container:SetSize(WIDTH, 300);
	DLUHQ.Panels.Tips.Container:SetPoint("TOPLEFT", DLUHQ.Panels.Tips, "TOPLEFT", 0, -PADDING_TOP);
	
	DLUHQ.Panels.Tips.Container.HelpText = DLUHQ.Panels.Tips.Container:CreateFontString(nil, "ARTWORK", "GameFontWhite");
	DLUHQ.Panels.Tips.Container.HelpText:SetPoint("TOPLEFT", DLUHQ.Panels.Tips.Container, "TOPLEFT", 8, -6);
	DLUHQ.Panels.Tips.Container.HelpText:SetJustifyH("LEFT");
	DLUHQ.Panels.Tips.Container.HelpText:SetWidth(WIDTH - (PADDING*2));
	
	CreateTips();

	-- Hook navigation
	DLUHQ.Character:SetScript("OnClick", function()
		togglePanel("Character");
	end);
	DLUHQ.LFR:SetScript("OnClick", function()
		togglePanel("LFR");
	end);
	DLUHQ.Tips:SetScript("OnClick", function()
		togglePanel("Tips");
	end);
	
	-- Navigate to front page by default
	if (DLU_UI["shownPanel"]) then
		togglePanel(DLU_UI["shownPanel"]);
	else
		togglePanel("Character");
	end
	
	CreateCurrencySelectorUI();
end

local function checkHQVisibility()
	if (DLU.player.level < 10 and not DLUSettings.showDLUHQOnLogon.value or not DLUSettings.showDLUHQOnLogon.value) then
		DLUHQ:Hide();
	end
end

-- Events
function events:LFG_UPDATE_RANDOM_INFO(...)
	local icons = {
		tank = DLU.iconToText("Interface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:16:16:0:0:64:64:0:19:22:41"),
		heal = DLU.iconToText("Interface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:16:16:0:0:64:64:20:39:1:20"),
		dps = DLU.iconToText("Interface\\LFGFrame\\UI-LFG-ICON-PORTRAITROLES:16:16:0:0:64:64:20:39:22:41")
	}

	createShortageList();

	local shortageAmount = #shortageList;
	if (shortageAmount > 0) then
		for i = 1, shortageAmount do
			local line = DLUHQ.Panels.LFR.ShortageContainer.Content.lines[i];

			local shortage = shortageList[i];
			local dunName = GetLFGDungeonInfo(shortage.dID);
			local text = dunName;

			if (shortage.tank) then
				text = format("%s %s", icons.tank, text);
			end
			if (shortage.heal) then
				text = format("%s %s", icons.heal, text);
			end
			if (shortage.dps) then
				text = format("%s %s", icons.dps, text);
			end

			line.text:SetText(text);
		end
	else
		local line = DLUHQ.Panels.LFR.ShortageContainer.Content.lines[1];
		line.text:SetText("No bonus rewards available at this moment...");
	end
end

function CreateTips()
	local mapID = C_Map.GetBestMapForUnit("player");
	
	local text = addonData.L.NoTips;
	
	if (mapID == 1469 or mapID == 1470) then
		local potionText = "%s potions is bad, then %s potion is good.";
		
		local potionOptions = {
			format(potionText, DLU.colorizeString("Black", "darkgray"), DLU.colorizeString("Green", "green")),
			format(potionText, DLU.colorizeString("Blue", "blue"), DLU.colorizeString("Purple", "purple")),
			format(potionText, DLU.colorizeString("Green", "green"), DLU.colorizeString("Red", "red")),
			format(potionText, DLU.colorizeString("Purple", "purple"), DLU.colorizeString("Black", "darkgray")),
			format(potionText, DLU.colorizeString("Red", "red"), DLU.colorizeString("Blue", "blue"))
		};
		
		text = "Horrific visions";
		
		for _,v in ipairs(potionOptions) do
			text = text .. "\n" .. v;
		end
		
		text = text .. "\n\nTo find the bad potion, you need to do following:\n1. Find a dead person next to a glowing note.\n2. Find the nearby potion.\n3. This is the bad potion, now remember it's color.";
		
		DLUHQ.Panels.Tips.Container.HelpText:SetText(text);
	 else
		local buff = AuraUtil.FindAuraByName("Winds of Wisdom", "player");
		if (buff ~= nil) then
			local buffText = "Winds of Wisdom is active, so you might want to level up your characters for maximum effect!";
			if (text == addonData.L.NoTips) then
				text = buffText;
			else
				text = text .. "\n\n" .. buffText;
			end
		end
		
		buff = AuraUtil.FindAuraByName("Impressive Influence", "player");
		if (buff ~= nil) then
			local buffText = "Impressive Influence is active, so you might want to get some reputation with Legion and BFA factions.";
			if (text == addonData.L.NoTips) then
				text = buffText;
			else
				text = text .. "\n\n" .. buffText;
			end
		end
	 
	 end
	
	DLUHQ.Panels.Tips.Container.HelpText:SetText(text);
end

function events:ARTIFACT_XP_UPDATE()
	if (DLU.player.level >= 100) then
		updateXpUI();
	end
end
function events:UNIT_INVENTORY_CHANGED()
	if (DLU.player.level >= 100) then
		updateXpUI();
	end
	if (DLUHQ.Panels.Character.ItemLevel:IsVisible()) then
		updateItemLevelUI();
	end
end
function events:HONOR_XP_UPDATE()
	updateHonorValue();
end
function events:PLAYER_PVP_RANK_CHANGED()
	--DLUHQ.Panels.Character.PvPBar:SetMinMaxValues(0, UnitHonorMax("player"));
end
function events:PLAYER_XP_UPDATE()
	updateXpValue();
end
function events:PLAYER_LEVEL_UP()
	--updateXpLabel();
	checkItemLevelUIDisplay();
end
function events:QUEST_REMOVED(questId)
	updateQuestUI();
end
function events:QUEST_ACCEPTED()
	updateQuestUI();
end
function events:UPDATE_EXHAUSTION()
	updateXpValue();
end
function events:AZERITE_ITEM_EXPERIENCE_CHANGED()
	updateAzeriteXpUI();
end
function events:CURRENCY_DISPLAY_UPDATE()
	updateCurrencies();
end
function events:ZONE_CHANGED_NEW_AREA()
	CreateTips();
end
function events:PLAYER_ENTERING_WORLD()
	TalentFrame_LoadUI(); -- Make sure the talentframe has been loaded
	hooksecurefunc("PlayerTalentFrame_Update", function(...)
		
		if (DLUTalentManagerBtn == nil) then
			local managerBtn = CreateFrame("Button", "DLUTalentManagerBtn", PlayerTalentFrame, BackdropTemplateMixin and "BackdropTemplate");
			managerBtn:SetPoint("BOTTOMRIGHT", PlayerTalentFrame.NineSlice.BottomRightCorner, "BOTTOMRIGHT", -8, 8);
			managerBtn.Label = addonData:CreateLabel("Open talent manager", 10, "CENTER", managerBtn);
			managerBtn:SetSize(16, 16);
			managerBtn:SetBackdrop(DLU_BD.secondary);
			managerBtn:SetBackdropColor(1,1,1,.2);
			managerBtn:SetBackdropBorderColor(1,1,1,0);
			
			managerBtn:SetSize(managerBtn.Label:GetWidth() + 4, 16);
			
			managerBtn:SetScript("OnEnter", hightlight);
			managerBtn:SetScript("OnLeave", unHightlight);
			managerBtn:SetScript("OnClick", function()
				DLUTalentFrame:Show();
			end);
		end
	end)
	
	CreateTalentUI();
end

-- Initialization
settingsFrame:RegisterEvent("PLAYER_LOGIN");
settingsFrame:SetScript("OnEvent", function()
	DLU_UI = DLU_UI or {};
	
	SetUpAddonOptions();
	createDLUHQUI();

	checkHQVisibility()

	eventFrame:SetScript("OnEvent", function(self, event, ...)
		events[event](self, ...);
	end);
	for k, v in pairs(events) do
		eventFrame:RegisterEvent(k);
	end
	
	settingsFrame:UnregisterEvent("PLAYER_LOGIN")
end);